package gus05.entity.gus.collection.map.filemap0.pw;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Filter;
import gus05.framework.features.Provide;
import gus05.framework.features.Register;
import gus05.framework.features.Retrieve;
import gus05.framework.features.Transform;
import gus05.framework.tools.DefaultSupport;



public class FileMap0Pw implements Entity, Transform, Register {


    public String getName()				{return "gus.collection.map.filemap0.pw";}
    public String getCreationDate()		{return "2008.06.28";}
    
    
    public static final long LAPSE = 500;
    
    private Service encrypt;
    private Service decrypt;
    
    private Transform current_encrypter;
    private Transform current_decrypter;
    
    private Timer timer;
    private TimerTask task;
    
    private HashSet set;
    
    
    public FileMap0Pw() throws Exception
    {
    	encrypt = Outside.service(this,"gus.security.encryption.pbe.stringcol.encrypt");
        decrypt = Outside.service(this,"gus.security.encryption.pbe.stringcol.decrypt");
        
        set = new HashSet();
    }
    
    
    
    
    public void register(String key, Object value) throws Exception
	{
		if(key.equals("password")) {initEncryption((String)value);return;}
		throw new Exception("Unknown key: "+key);
	}
    
    
    public Object transform(Object obj) throws Exception
    {
    	File f = (File)obj;
    	if(current_encrypter==null) throw new Exception("encryption has not been initialized");
    	if(current_decrypter==null) throw new Exception("encryption has not been initialized");
    	
    	if(timer==null) initTimer();
    	return new Map0(f,current_decrypter,current_encrypter);
    }
    
    
    
    
    
    
    
    
    private void initEncryption(String password)
    {
    	try
    	{
    		current_encrypter = (Transform) encrypt.transform(password);
    		current_decrypter = (Transform) decrypt.transform(password);
    	}
    	catch(Exception e)
    	{Outside.err(this,"initEncryption(String)",e);}
    }
    
    
    
    
    private void initTimer()
    {
    	timer = new Timer();
    	task = new TimerTask(){
			public void run() {saveAll();}
		};
		timer.schedule(task,new Date(),LAPSE);
    }
    
    

    
    
    
    private synchronized void saveAll()
    {
    	Iterator it = set.iterator();
    	while(it.hasNext())
    	{
    		Map0 m = (Map0) it.next();
    		m.saveMap();
    	}
    	set.clear();
    }
    
    
    
    private synchronized void waitingForSave(Map0 m)
    {
    	set.add(m);
    }
    

    
    
    private void saveMap(File file, Map map, Transform encrypter)
    {
        try
        {
        	map = (Map) encrypter.transform(map);
        	
        	Properties prop = new Properties();
            prop.putAll(map);
        	FileOutputStream fos = new FileOutputStream(file);
            prop.store(fos,"");
            fos.close();
        }
        catch(Exception e)
        {Outside.err(this,"saveMap(File,Map,Transform)",e);}
    }
    
    
    
    
    private Map loadMap(File file, Transform decrypter) throws Exception
    {
    	Properties prop = new Properties();
    	if(file.exists())
    	{
    		FileInputStream fis = new FileInputStream(file);
            prop.load(fis);
            fis.close();
    	}
        return (Map) decrypter.transform(prop);
    }
    
    
    
    
    
    
    
    
    
    
    
    

    

    
    
    private class Map0 extends DefaultSupport implements Map, Retrieve, Register, Transform, Provide, Filter
    {
    	private Transform encrypter;
    	
        private File file;
        private Map map;
        
        private String keyName = null;
        private String valueName = null;
        
        private Class keyClass = null;
        private Class valueClass = null;
        
        
        private Object lastElement;
        private Object lastResult;
        private Object lastKey;
        
        
        
        public void saveMap()
        {FileMap0Pw.this.saveMap(file, map, encrypter);}
        
        
        
        
        
        public Map0(File file, Transform decrypter, Transform encrypter) throws Exception
        {
            this.file = file;
            this.encrypter = encrypter;
            map = loadMap(file,decrypter);
        }
        
        public int size()                           {return map.size();}
        public boolean isEmpty()                    {return map.isEmpty();}
        public boolean containsKey(Object key)      {return map.containsKey(key);}
        public boolean containsValue(Object value)  {return map.containsValue(value);}
        public Set keySet()                         {return map.keySet();}
        public Collection values()                  {return map.values();}
        public Set entrySet()                       {return map.entrySet();}
        public Object get(Object key)               {return map.get(key);}
        public String toString()                    {return map.toString();}
        public boolean equals(Object o)             {return map.equals(o);}
        
        
        
        public Object put(Object key, Object value)
        {
            lastKey = key;
            lastElement = value;
            lastResult = map.put(key,value);
            
            waitingForSave(this);
            send(this,"put(Object,Object)");
            return lastResult;
        }

        
        public Object remove(Object key)
        {
            lastKey = key;
            lastResult = map.remove(key);
            
            waitingForSave(this);
            send(this,"remove(Object)");
            return lastResult;
        }
        

        public void putAll(Map m)
        {
            map.putAll(m);
            waitingForSave(this);
            send(this,"putAll(Map)");
        }

        
        public void clear()
        {
            map.clear();
            waitingForSave(this);
            send(this,"clear()");
        }
        
        

        public Object retrieve(String key) throws Exception
        {
            if(key.equals("lastElement"))return lastElement;
            if(key.equals("lastResult"))return lastResult;
            if(key.equals("lastKey"))return lastKey;
            if(key.equals("map"))return map;
            
            if(key.equals("keyName"))return keyName;
            if(key.equals("valueName"))return valueName;
            if(key.equals("keyClass"))return keyClass;
            if(key.equals("valueClass"))return valueClass;
            
            if(key.equals("keys"))
                return new String[]{"lastElement","lastResult","lastKey","map","keyName","valueName","keyClass","valueClass"};
            
            throw new Exception("unknown key "+key);
        }


        public void register(String key, Object obj) throws Exception
        {
            if(key.equals("keyName"))       {keyName = (String)obj;return;}
            if(key.equals("valueName"))     {valueName = (String)obj;return;}
            if(key.equals("keyClass"))      {keyClass = (Class)obj;return;}
            if(key.equals("valueClass"))    {valueClass = (Class)obj;return;}
            
            throw new Exception("unknown key "+key);
        }

        public boolean filter(Object obj) throws Exception      {return map.containsKey(obj);}
        public Object transform(Object obj) throws Exception    {return map.get(obj);}
        public Object provide() throws Exception                {return map;}
    }
}

