package gus05.manager.gus.endorf.layer3.loading;

import gus05.manager.gus.endorf.layer3.system1.System1;
import gus05.manager.gus.endorf.layer3.tools.Tools_file;
import gus05.manager.gus.endorf.layer3.tools.Tools_fileFilter;

import java.io.File;
import java.util.Iterator;
import java.util.Locale;
import java.util.Properties;
import java.util.Set;


public class Loader_lings extends Loader {

	
	
	public static boolean isISO639_LanguageCode(String code)
    {
    	String[] codes = Locale.getISOLanguages();
    	for(int i=0;i<codes.length;i++)
    		if(codes[i].equals(code)) return true;
    	return false;
    }
	
	
	
	
    
    public Loader_lings()
    {super("lings");}
    
    
    protected void insideLoad()
    {
        try
        {
            Set set = insideDir("ling");
            if(set==null) return;
            
            Iterator it = set.iterator();
            while(it.hasNext())
            {
                String name = (String)it.next();
                Properties p0 = insideProp("ling/"+name);
                if(p0!=null) putLingProperties(name,p0);
            }
            debug("insideLoad()","load "+set.size()+" ling properties from inside");
        }
        catch(Exception e)
        {System1.handleError(this,"insideLoad()",e);}
    }
    
    
    
    
    
    protected void outsideLoad()
    {
        try
        {           
            File lingDir = outsideDir(System1.PATH_LINGDIR);
            if(lingDir==null)return;
            
            File[] f = lingDir.listFiles(Tools_fileFilter.lingFileFilter);
            debug("outsideLoad()","load "+f.length+" lings from: "+lingDir.getAbsolutePath());
            for(int i=0;i<f.length;i++)loadFromFile(f[i]);
        }
        catch(Exception e)
        {System1.handleError(this,"outsideLoad()",e);}
    }
    
    
    
    
    private void loadFromFile(File f)
    {
        try
        {
            Properties prop = Tools_file.prop(f);
            putLingProperties(f.getName(),prop);
        }
        catch(Exception e)
        {System1.handleError(this,"loadFromFile(File)",e);}
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    private void putLingProperties(String name, Properties prop)
    {
        String ext = findLanguageExtension(name);
        if(!isISO639_LanguageCode(ext))
        {
        	warning("putLingProperties(String,Properties)","invalid ling resource name: "+name);
        	return;
        }
        Iterator it = prop.keySet().iterator();
        while(it.hasNext())
        {
            String key = (String)it.next();
            String value = prop.getProperty(key);
            putLing(key+"_"+ext,value);
        }
    }
    
    
    private void putLing(String key, String value)
    {
        if(System1.lingMap.containsKey(key))
            warning("putLing(String,String)","ling key: "+key+" is loaded many times");
        System1.lingMap.put(key,value);
        increase();
    }
    
    
    
    private String findLanguageExtension(String value)
    {
        String name = getFirstPart(value);
        String[] n = name.split("_",-1);
        return n[n.length-1];
    }
    
    
    
    private String getFirstPart(String value)
    {
        if(!value.contains(".")) return value;
        
        String[] n = value.split("\\.");
        int l = n[n.length-1].length();
        return value.substring(0,value.length()-1-l);
    }
    

}
